/*******************************************************************************
 * Copyright 2022 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef MODALCV_WARP_H
#define MODALCV_WARP_H


#ifdef __cplusplus
extern "C" {
#endif


#include <stdint.h>

//For lens parameters struct, should probable be moved to a common file
#include "modalcv/undistort_cpu.h"

////////////////////////////////////////////////////////////////////////////////
// CVP Warp
////////////////////////////////////////////////////////////////////////////////

typedef void* mcv_warp_handle;

typedef enum
{
    WARP_GRID = 0x1,
    WARP_PERSPECTIVE = 0x2,
    WARP_PERSPECTIVE_GRID = 0x3,
} mcvWarpMode;

typedef struct mcv_warp_config_t{
    int width;
    int height;
    float zoom;
    mcvWarpMode mode;
    mcv_lens_params_t lens;
    float transform[9];
} mcv_warp_config_t;


mcv_warp_handle mcv_warp_init(mcv_warp_config_t config);
int mcv_warp_deinit(mcv_warp_handle handle);

// disparity and occlusion masks are in a 4x4 raster, use other functions to
// convert to other desired output formats
int mcv_warp_process(mcv_warp_handle handle, uint8_t* inImg, uint8_t **outImg);



#ifdef __cplusplus
}
#endif

#endif // end #define MODALCV_UNDISTORT_CPU_H
